/*
 * plugin.c: 
 * Handles dynamically loaded modules.
 *
 * Copyright (C) 2002 Sakari Lehtonen <sakari@ionstream.fi>
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include "plugin.h"

static void *load_dyn_module(const char *path);

/* the growing array that holds information about all available
 * plugins (name, path) */
static nvtab plugintab =
{
	0,
	0,
	NULL
};

/* holds information about loaded plugins (name, handle) */
static nvtab loadedtab =
{
	0,
	0,
	NULL
};

/* initialize the plugintab to contain all plugin names and paths to
 * the actual files */
int init_plugins(void)
{
	scan_plugin_dir(globalopts.plugindir);
	if( strcmp(globalopts.plugindir, DEF_PLUGIN_DIR) != 0 )
		scan_plugin_dir(DEF_PLUGIN_DIR);

	return 0;
}

void list_plugins(void)
{
	int i;

	printf(_("Plugins found in dir(s) '%s'"), 
		globalopts.plugindir);	
	if( strcmp(globalopts.plugindir, DEF_PLUGIN_DIR) != 0 )
		printf(", '%s'", DEF_PLUGIN_DIR);
	printf(":\n");
	for(i=0; i<plugintab.nval; i++)
	{
		printf("%s\n", plugintab.nv[i].name);
	}
}

/* adds the plugin to plugintab */
int add_plugin(const char *path, const char *basename)
{
	nameval plugin;
	char *ext;

	if( (ext = strrchr(basename, '.')) != NULL )
	{
		/* TODO: ok the extension is not always '.so', fix maybe */
		if( strncmp(ext, ".so", 3) == 0 )
		{
			plugin.name = strdup(basename);
			plugin.val = (char *)strdup(path);
			addnv(&plugintab, plugin);
		}
	}

	return 0;
}

/* scans the contents of dirname and adds all plugins in that dir
 * to plugintab */
int scan_plugin_dir(const char *dirname)
{
	DIR *dir;
	struct dirent *dirent;
	struct stat sbuf;
	char path[STRLEN];

	if( (dir = opendir(dirname)) == NULL )
		return -1;

	/* scan the directory contents */
	while( (dirent = readdir(dir)) != NULL )
	{
		/* dont process files beginning with '.' */
		if( dirent->d_name[0] == '.' )
			continue;

		snprintf(path, sizeof(path), "%s/%s", 
			dirname, dirent->d_name);
		if( stat(path, &sbuf) == 0 )
		{
			if( S_ISREG(sbuf.st_mode) == 1 )
			{
				/* add the plugin if it has not yet been
				 * loaded */
				if( lookupbyname(&plugintab, 
					dirent->d_name) == NULL )
					add_plugin(path, dirent->d_name);
			}
		}
	}
	closedir(dir);

	return 0;
	
}

/* loads and returns an opened dynamic module handle */
void *load_plugin(const char *name, int type)
{
	nameval plugin;
	void *lookup;
	void *handle;
	char basename[STRLEN/2];
	char *format;
	char *path;
	int ch, i;

	/* try to look if the module has been already loaded */
	if( (lookup = lookupbyname(&loadedtab, name)) != NULL )
		return (void *)lookup;

	/* what kind of plugin are we loading */
	if( type == PLUGIN_DRIVER )
		format = "libmixer%s.so";
	else
		format = "libui%s.so";
	snprintf(basename, sizeof(basename), format, name);

	/* convert to lowercase */
	for(i=0; i<strlen(basename); i++)
	{
		ch = basename[i];
		basename[i] = tolower(ch);
	}
	/* search for the plugin in the plugintab that contains all
	 * detected plugins and their paths */
	lookup = lookupbyname(&plugintab, basename);
	if( lookup == NULL )
	{
		errormsg(_("plugin %s not found!"), basename);
		return NULL;
	}
	/* get the path and try to load the module */
	path = (char *)lookup;
	if( (handle = load_dyn_module(path)) == NULL )
		return NULL;

	/* add the loaded plugin to the loadedtab */
	plugin.name = strdup(name);
	plugin.val = (void *)handle;
	addnv(&loadedtab, plugin);

#ifdef UMIX_DEBUG
	errormsg("load_plugin: plugin %s loaded, basename=%s path=%s", 
		name, basename, (char *)lookup);
#endif

	return handle;
}

void *get_dyn_module_sym(void *handle, const char *symbol)
{
	return dlsym(handle, symbol);
}

/* loads and returns the opened dynamic module from path */
static void *load_dyn_module(const char *path)
{
	void *handle;

	handle = dlopen(path, RTLD_NOW);
	if( handle == NULL )
	{
		errormsg("%s: %s", dlerror());
		return NULL;
	}
	
	return handle;
}
