/*
 * ui_gtk.c: 
 * GTK+ user interface for Umix.
 *
 * Copyright (C) 2002 Sakari Lehtonen <sakari@ionstream.fi>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <gtk/gtk.h>
#include "umix_common.h"
#include "ui.h"
#include "prefs.h"
#include "support.h"
#include "umix-logo.xpm"
/*
#include "spkr_left.xpm"
#include "spkr_right.xpm"
*/

#define UMIX_UI_GTK_VERSION "0.1"

/* TODO:
 *
 * Before 0.2:
 * - Selecting which channels to show
 * - Selecting the config file to save / load settings
 * - Saving configuration options to umixrc .. section [gtk]
 * - Preferences dialog
 * - Selecting which channel the bal adj controls 
 * - Hiding the record vboxes
 * - Check the pixmap code
 * - Check that is fully compatible with GTK+2.0, this is converted from 1.4
 *   code
 * - Fix the recursion that happens when update_sliders is called
 *
 * Later:
 * - Entering volumes manually with Ctrl-e or something
 * - Find a way of focusing single channels
 * - Multiple mixer support
 * - Maybe split the code .. menu stuff in menu.c etc
 * - Check out support.[ch] and unify coding style .. 
 * - Clean up the code
 *
 */

enum ADJUSTMENT_DEFS
{
	ADJ_BAL_MIN = -VOL_MAX,
	ADJ_BAL_MAX = VOL_MAX,
	ADJ_BAL_STEP_INCR = 1,
	ADJ_BAL_PAGE_INCR = VOL_MAX,
	ADJ_BAL_PAGE_SIZE = 0,
	ADJ_VOL_MIN = -VOL_MAX,
	ADJ_VOL_MAX = 0,
	ADJ_VOL_STEP_INCR = VOL_MAX/10,
	ADJ_VOL_PAGE_INCR = VOL_MAX/10,
	ADJ_VOL_PAGE_SIZE = 0,
	DEF_UPDATE_TIMEOUT = 1000
};

/* global variables */
GtkWidget *top_window;

/* the channel number that the balance adj controls */
static int balance_channel = 0;

/* the list of visible channels */
static int *visible_channels[MAX_MIXERS];

static GtkAdjustment **volume_adj[MAX_MIXERS];
static GtkAdjustment *balance_adj;
/* hmm, this keeps track of our own volume updates .. maybe figure out
 * a better way */
static int own_update = 0;

static GtkWidget *top_vbox;
static GtkWidget *statusbar;
static GtkWidget *channel_hbox;
static GtkWidget *balance_hbox;
static GtkWidget *menu_bar;
static GtkWidget *about_window = NULL;

#if 0
gtk_prefs gtk_config =
{
	TRUE, /* show_record_button */
	TRUE, /* show_balance_adj */
	TRUE  /* update_sliders */
};
#endif 

/* global function prototypes */
int ui_gtk_main(void);
void ui_gtk_exit(void);
int ui_gtk_check(void);

/* internal function prototypes */
static GtkWidget *create_top_window(void);
static GtkWidget *create_menu(GtkAccelGroup *accel_group);
static GtkWidget *create_channel_vbox(void);
static GtkWidget *create_channels_hbox(void);
static GtkWidget *create_balance_hbox(void);
static GtkWidget *create_record_vbox(void);

static void push_statusbar(gint context_id, const char *fmt, ...);
static void init_stuff(void);
static void create_stuff(void);
static gint update_sliders(gpointer data);

/* callback functions */
static void cb_vol_adj_chg(GtkAdjustment *adj, gpointer data);
static void cb_bal_adj_chg(GtkAdjustment *adj, gpointer data);
static void cb_rec_toggled(GtkToggleButton *button, gpointer data);
static void cb_mute_toggle(GtkWidget *widget, gpointer data);

static void cb_quit_func(GtkButton *button, gpointer *data);

static void cb_show_about(GtkWidget *widget, gpointer data);
static void cb_about_delete(GtkWidget *widget, gpointer data);

ui_func gtk_func =
{
	ui_gtk_main,
	ui_gtk_exit,
	ui_gtk_check
};

ui_func *get_ui_func(void)
{
	return &gtk_func;
}

int ui_gtk_main(void)
{
	/* initialize the environment */
	gtk_init(NULL, NULL);

	mixer_set_curr(0);
	chan_set_curr(0);

	/* create widgets */
	top_window = create_top_window();
	/* the top vbox contains everything */
	top_vbox = gtk_vbox_new(NONHOMOGENEOUS, 0);
	gtk_container_add(GTK_CONTAINER(top_window), top_vbox);
	gtk_widget_show(top_vbox);

	init_stuff();
	create_stuff();
	/*
	update_stuff();
	*/
	
	/* hmm not sure if this is the right way to do this .. */
	/* add timeout to check if the mixer is updated and update the
	 * sliders */
	/*
	if( gtk_config.update_sliders == TRUE )
	*/
	gtk_timeout_add(DEF_UPDATE_TIMEOUT, update_sliders, NULL);

	/* show */
	gtk_widget_show(top_window);
	gtk_main();

	return 0;
}

/* TODO: rename, fix, and check */
static void create_stuff(void)
{
	GtkAccelGroup *accel_group;
	gint context_id;
	
	/* set up the accelerator group */
	accel_group = gtk_accel_group_new();
	gtk_window_add_accel_group(GTK_WINDOW(top_window), accel_group);
	
	/* create the menu bar */
	menu_bar = create_menu(accel_group);
	gtk_box_pack_start(GTK_BOX(top_vbox), menu_bar, NOEXPAND, NOFILL, 0);
	gtk_widget_show(menu_bar);
	
	/* the channel_hbox contains all the channel vboxes */
	channel_hbox = create_channels_hbox();
	gtk_box_pack_start(GTK_BOX(top_vbox), channel_hbox, EXPAND, FILL, 0);
	gtk_widget_show(channel_hbox);

	/* create the balance adjustment */
	balance_hbox = create_balance_hbox();
	gtk_box_pack_start(GTK_BOX(top_vbox), balance_hbox, NOEXPAND,
		NOFILL, 0);
	gtk_widget_show(balance_hbox);
	
	/* status bar stuff */
	statusbar = gtk_statusbar_new();
	context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar),
		"main");
	push_statusbar(context_id, "%s :: %s", mixer_get_name(), 
		mixer_get_path());
	gtk_box_pack_start(GTK_BOX(top_vbox), statusbar, NOEXPAND,
		NOFILL, 0);
	gtk_widget_show(statusbar);
}

/* TODO: rename, check and prehaps move to prefs.c .. */
/*
void update_stuff(void)
{
	if( gtk_config.show_balance_adj == TRUE )
		gtk_widget_show(balance_hbox);
	else
		gtk_widget_hide(balance_hbox);
}
*/

/* TODO: rename, check, and FIX */
static void init_stuff(void)
{
	int i;
	int numchan;

	numchan = mixer_get_num_chan();
	volume_adj[0] = emalloc(sizeof(GtkAdjustment) * numchan);
	visible_channels[0] = emalloc(sizeof(int) * numchan + 1);
	for(i=0; i<mixer_get_num_chan(); i++)
	{
		chan_set_curr(i);
		visible_channels[0][i] = 1;
	}
}

static void cb_vol_adj_chg(GtkAdjustment *adj, gpointer data)
{
	float vol;
	gint context_id;

	own_update = 1;
	chan_set_curr(GPOINTER_TO_INT(data));
	vol = ABS(adj->value);
#ifdef UMIX_DEBUG
	errormsg("cb_vol_adj_chg: ch=%d value=%f", 
		chan_get_curr(), adj->value);
#endif
	context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar),
		"vol_adj");
	gtk_statusbar_pop(GTK_STATUSBAR(statusbar), context_id);
	chan_set_vol(vol);
	push_statusbar(context_id, "Volume for channel %s set to %1.f", 
		chan_get_name(), vol);
}

static void cb_bal_adj_chg(GtkAdjustment *adj, gpointer data)
{
	float bal;
	gint context_id;

	own_update = 1;
	chan_set_curr(GPOINTER_TO_INT(data));
	bal = adj->value/VOL_MAX;
#ifdef UMIX_DEBUG
	errormsg("cb_bal_adj_chg: ch=%d value=%f", 
		chan_get_curr(), adj->value);
#endif
	context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar),
		"bal_adj");
	gtk_statusbar_pop(GTK_STATUSBAR(statusbar), context_id);
	chan_set_bal(bal);
	push_statusbar(context_id, "Balance for channel %s set to %1.f", 
		chan_get_name(), bal*VOL_MAX);
}

static void cb_rec_toggled(GtkToggleButton *button, gpointer data)
{
#ifdef UMIX_DEBUG
	errormsg("cb_rec_toggled: data=%d state=%d", GPOINTER_TO_INT(data),
		gtk_toggle_button_get_active(button));
#endif
	chan_set_curr(GPOINTER_TO_INT(data));
	if( gtk_toggle_button_get_active(button) == TRUE )
		chan_set_rec(CHAN_RECSRC);
	else
		chan_set_rec(CHAN_PLAY);
}

/* TODO: check, esp multiple mixer support */
static void cb_mute_toggle(GtkWidget *widget, gpointer data)
{
	int i;
	int orig_chan;
	gint context_id;

	orig_chan = chan_get_curr();
	for(i=0; i<mixer_get_num_chan(); i++)
	{
		chan_set_curr(i);
		if( visible_channels[0][i] == 1 )
		{
			if( chan_is_mute() == 0 )
				chan_set_mute(MUTE);
			else
				chan_set_mute(UNMUTE);
			
			/* adjust the slider */
			gtk_adjustment_set_value(volume_adj[0][i], 
				-chan_get_vol());
		}
	}
	context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar),
		"toggle_mute");
	gtk_statusbar_pop(GTK_STATUSBAR(statusbar), context_id);
	push_statusbar(context_id, "Mute");

	chan_set_curr(orig_chan);
}

static gint update_sliders(gpointer data)
{
	int i;
	int orig_chan;

	/* own_update tracks if the current process of umix has set 
	 * the volume last, so we don`t hopefully update the volumes twice */
	if( own_update == 0 && mixer_get_status() == 1 )
	{
		orig_chan = chan_get_curr();
		for(i=0; i<mixer_get_num_chan(); i++)
		{
			chan_set_curr(i);
			if( visible_channels[0][i] == 1 )
			{
				/* adjust the slider */
				gtk_adjustment_set_value(volume_adj[0][i], 
					-chan_get_vol());
			}
		}
		/* not enabled ATM .. some problems with recursion 
		if( gtk_config.show_balance_adj == TRUE )
		{
			chan_set_curr(balance_channel);
			gtk_adjustment_set_value(bal_adj, chan_get_bal()*100.0);
		}
		*/
		chan_set_curr(orig_chan);
	}
	own_update = 0;

	return TRUE;
}

static void push_statusbar(gint context_id, const char *fmt, ...)
{
	va_list ap;
	char buf[STRLEN];

	va_start(ap, fmt);
	vsnprintf(buf, sizeof(buf), fmt, ap);
	va_end(ap);
	
	gtk_statusbar_push(GTK_STATUSBAR(statusbar), context_id, buf);
}

/* fill the channel_hbox and record_hbox */
static GtkWidget *create_channels_hbox(void)
{
	GtkWidget *top_hbox;
	GtkWidget *top_vbox;
	GtkWidget *chan_vbox;
	GtkWidget *rec_vbox;
	GtkWidget *vsep;
	int i;
	int numchan;

	top_hbox = gtk_hbox_new(NONHOMOGENEOUS, 6);
	gtk_container_set_border_width(GTK_CONTAINER(top_hbox), 2);

	/* TODO: add option what channels to show .. like a list of
	 * what channels to show, and fix the last vseparator not to be 
	 * add following options to be selectable :
	 *
	 * - Show only channels that are not muted 
	 * etc ..
	 */
	
	numchan = mixer_get_num_chan();
	for(i=0; i<numchan; i++)
	{
		chan_set_curr(i);
		if( visible_channels[0][i] == 1 )
		{
			top_vbox = gtk_vbox_new(NONHOMOGENEOUS, 6);
			gtk_widget_show(top_vbox);

			/* the channel vbox */
			chan_vbox = create_channel_vbox();
			gtk_box_pack_start(GTK_BOX(top_vbox),
				chan_vbox, NOEXPAND, NOFILL, 0);
			gtk_widget_show(chan_vbox);
			
			/* the record vbox */
			rec_vbox = create_record_vbox();
			gtk_box_pack_start(GTK_BOX(top_vbox), rec_vbox,
				NOEXPAND, NOFILL, 0);
			gtk_widget_show(rec_vbox);
			
			gtk_box_pack_start(GTK_BOX(top_hbox), top_vbox,
				NOEXPAND, NOFILL, 0);
			
			/* TODO: don`t draw the last vseparator */
			vsep = gtk_vseparator_new();
			gtk_box_pack_start(GTK_BOX(top_hbox), vsep, 
				EXPAND, FILL, 0);
			gtk_widget_show(vsep);
		}
	}

	return top_hbox;
}

#if 0
static GtkWidget *create_balance_hbox(void)
{
	GtkWidget *top_hbox;
	GtkWidget *scale;
	GtkWidget *right_pixmap, *left_pixmap;
	
	top_hbox = gtk_hbox_new(NONHOMOGENEOUS, 0);

	left_pixmap = pixmap_create_from_d(top_window, spkr_left_xpm);
	gtk_box_pack_start(GTK_BOX(top_hbox), left_pixmap, NOEXPAND, NOFILL, 4);
	gtk_widget_show(left_pixmap);
	
	right_pixmap = pixmap_create_from_d(top_window, spkr_right_xpm);
	gtk_box_pack_start(GTK_BOX(top_hbox), right_pixmap, NOEXPAND,
		NOFILL, 4);
	gtk_widget_show(right_pixmap);

	/* init the balance values */
	chan_set_curr(balance_channel);
	balance_adj = GTK_ADJUSTMENT(gtk_adjustment_new(chan_get_bal()*VOL_MAX, 
				ADJ_BAL_MIN, ADJ_BAL_MAX, ADJ_BAL_STEP_INCR,
				ADJ_BAL_PAGE_INCR, ADJ_BAL_PAGE_SIZE));
	gtk_signal_connect(GTK_OBJECT(balance_adj), "value_changed",
		(GtkSignalFunc)cb_bal_adj_chg, (gpointer)balance_channel);
	scale = gtk_hscale_new(GTK_ADJUSTMENT(balance_adj));
	gtk_scale_set_draw_value(GTK_SCALE(scale), FALSE);
	gtk_box_pack_start(GTK_BOX(top_hbox), scale, EXPAND, FILL, 0);
	gtk_widget_show(scale);


	return top_hbox;
}
#endif

static GtkWidget *create_balance_hbox(void)
{
	GtkWidget *top_hbox;
	GtkWidget *scale;
	
	top_hbox = gtk_hbox_new(NONHOMOGENEOUS, 0);

	/* init the balance values */
	chan_set_curr(balance_channel);
	balance_adj = GTK_ADJUSTMENT(gtk_adjustment_new(chan_get_bal()*VOL_MAX, 
				ADJ_BAL_MIN, ADJ_BAL_MAX, ADJ_BAL_STEP_INCR,
				ADJ_BAL_PAGE_INCR, ADJ_BAL_PAGE_SIZE));
	gtk_signal_connect(GTK_OBJECT(balance_adj), "value_changed",
		(GtkSignalFunc)cb_bal_adj_chg, (gpointer)balance_channel);
	scale = gtk_hscale_new(GTK_ADJUSTMENT(balance_adj));
	gtk_scale_set_draw_value(GTK_SCALE(scale), FALSE);
	gtk_box_pack_start(GTK_BOX(top_hbox), scale, EXPAND, FILL, 0);
	gtk_widget_show(scale);

	return top_hbox;
}

/* creates and returns the vbox that holds all widgets for one channel */
static GtkWidget *create_channel_vbox(void)
{
	GtkWidget *top_vbox;
	GtkWidget *chan_label, *vol_label;
	GtkWidget *vol_scale;
	GtkWidget *hsep;
	int curr_chan = chan_get_curr();

	top_vbox = gtk_vbox_new(NONHOMOGENEOUS, 0);

	/* channel label */
	chan_label = gtk_label_new(chan_get_label());
	gtk_misc_set_alignment(GTK_MISC(chan_label), 0.5, 0);
	gtk_box_pack_start(GTK_BOX(top_vbox), chan_label, 
		NOEXPAND, NOFILL, 0);
	gtk_widget_show(chan_label);

	hsep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(top_vbox), hsep, EXPAND, FILL, 0);
	gtk_widget_show(hsep);

	/* TODO: maybe add option to show channel icons */
	vol_label = gtk_label_new(_("Volume:"));
	gtk_misc_set_alignment(GTK_MISC(vol_label), 0.5, 0);
	gtk_box_pack_start(GTK_BOX(top_vbox), vol_label, NOEXPAND,
		NOFILL, 0);
	gtk_widget_show(vol_label);

	/* store all the volume adjustments global so we can update them
	 * when the volume changes  */
	volume_adj[0][curr_chan] = GTK_ADJUSTMENT(
				gtk_adjustment_new(-chan_get_vol(),
				ADJ_VOL_MIN, ADJ_VOL_MAX, ADJ_VOL_STEP_INCR,
				ADJ_VOL_PAGE_INCR, ADJ_VOL_PAGE_SIZE));
	gtk_signal_connect(GTK_OBJECT(volume_adj[0][curr_chan]),"value_changed",
		(GtkSignalFunc)cb_vol_adj_chg, (gpointer)chan_get_curr());
	vol_scale = gtk_vscale_new(GTK_ADJUSTMENT(volume_adj[0][curr_chan]));
	gtk_scale_set_draw_value(GTK_SCALE(vol_scale), FALSE);
	gtk_widget_set_usize(vol_scale, -2, VOL_MAX);
	gtk_box_pack_start(GTK_BOX(top_vbox), vol_scale, EXPAND,
		FILL, 0);
	gtk_widget_show(vol_scale);

	return top_vbox;
}

/* creates and returns a recbutton box for one channel */
static GtkWidget *create_record_vbox(void)
{
	GtkWidget *top_vbox;
	GtkWidget *button_vbox;
	GtkWidget *hsep;
	GtkWidget *rec_radiobutton;
	static GSList *rec_group = NULL;

	top_vbox = gtk_vbox_new(NONHOMOGENEOUS, 0);
	
	hsep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(top_vbox), hsep, EXPAND, FILL, 0);
	gtk_widget_show(hsep);
	
	button_vbox = gtk_vbox_new(NONHOMOGENEOUS, 0);
	/* the rec check button, if channel can be a recording source */
	if( chan_is_record() )
	{
		gtk_box_pack_start(GTK_BOX(top_vbox), button_vbox, 
			NOEXPAND, NOFILL, 0);

		rec_radiobutton = gtk_radio_button_new_with_label(rec_group, 
			"Rec");
		gtk_signal_connect(GTK_OBJECT(rec_radiobutton), 
			"toggled", (GtkSignalFunc)cb_rec_toggled, 
			(gpointer)chan_get_curr());
		rec_group = gtk_radio_button_group(GTK_RADIO_BUTTON(
				rec_radiobutton));
		if( chan_is_recsrc() )
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(
					rec_radiobutton), TRUE); 
		gtk_box_pack_start(GTK_BOX(button_vbox), 
			rec_radiobutton, NOEXPAND, NOFILL, 0);
		gtk_widget_show(rec_radiobutton);

	}
	/* pad with empty .. hmm not sure about this */
	else
		gtk_box_pack_start(GTK_BOX(top_vbox), button_vbox, 
			EXPAND, FILL, 12);
	
	gtk_widget_show(button_vbox);
	return top_vbox;
}

/* creates and returns the top menu */
static GtkWidget *create_menu(GtkAccelGroup *accel_group)
{
	GtkWidget *menu;
	GtkItemFactory *item_fact;
	GtkItemFactoryEntry items[] =
	{
		{ "/File", NULL, NULL, 0, "<Branch>" },
		{ "/File/sep1", NULL, NULL, 0, "<Separator>" },
		{ "/File/_Quit","<Ctrl>Q", cb_quit_func, 0, "<Item>" },
		/*
		{ "/Settings", NULL, NULL, 0, "<Branch>" },
		{ "/Settings/_Preferences...","<Ctrl>P",cb_show_prefs,0,
			"<Item>" },
		*/
		{ "/Mute", NULL, NULL, 0, "<Branch>" },
		{ "/Mute/_Mute All", "M", cb_mute_toggle, 0, "<CheckItem>" },
		{ "/Help", NULL, NULL, 0, "<Branch>" },
		{ "/Help/sep2", NULL, NULL, 0, "<Separator>" },
		{ "/Help/About", NULL, cb_show_about, 0, "<Item>" }
	};

	/* create the item factory */
	item_fact = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", 
		accel_group);
	gtk_item_factory_create_items(item_fact, NELEMS(items), items, NULL);
	menu = gtk_item_factory_get_widget(item_fact, "<main>");

	return menu;
}

static void cb_about_delete(GtkWidget *widget, gpointer data)
{
	gtk_widget_destroy(about_window);
	about_window = NULL;
}       

static void cb_show_about(GtkWidget *widget, gpointer data)
{
        GtkWidget *frame;
        GtkWidget *vbox;
        GtkWidget *label;
	GtkWidget *logo;
	char buf[BUFLEN];

        if (about_window != NULL)
	{
		gdk_window_raise(about_window->window);
		return;
	}

        about_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
        gtk_window_set_wmclass(GTK_WINDOW(about_window), "about", "Umix");
        gtk_signal_connect(GTK_OBJECT(about_window), "delete_event", 
		GTK_SIGNAL_FUNC(cb_about_delete), NULL);

        gtk_window_set_title(GTK_WINDOW(about_window), "About - Umix");
	gtk_window_set_policy(GTK_WINDOW(about_window), 
		NOSHRINK, NOGROW, NOAUTOGROW);
	gtk_window_set_position(GTK_WINDOW(about_window), GTK_WIN_POS_MOUSE);
        gtk_container_border_width(GTK_CONTAINER(about_window), 5);

        frame = gtk_frame_new(NULL);
        gtk_container_add(GTK_CONTAINER(about_window), frame);
	gtk_widget_show(frame);

        vbox = gtk_vbox_new(NONHOMOGENEOUS, 10);
        gtk_container_border_width(GTK_CONTAINER(vbox), 5);
        gtk_container_add(GTK_CONTAINER(frame),vbox);
	gtk_widget_show(vbox);
	
	logo = pixmap_create_from_d(top_window, umix_logo_xpm);
	gtk_box_pack_start(GTK_BOX(vbox), logo, NOEXPAND, NOFILL, 0);
	gtk_widget_show(logo);
	
	snprintf(buf, sizeof(buf), 
		"%s - GTK+ UI v%s\n\n"
		_("A program for adjusting soundcard volumes\n"
		"and other features in soundcard mixers.\n\n"
		"Copyright (C) 2002 Sakari Lehtonen\n"
		"http://www.ionstream.fi/sakari/umix/\n")
		, get_version_str(), UMIX_UI_GTK_VERSION);

        label = gtk_label_new(buf);
        gtk_box_pack_start(GTK_BOX(vbox), label, NOEXPAND, NOFILL, 0);
        gtk_widget_show(label);

	gtk_widget_show(about_window);
}

/* creates and returns the top window */
static GtkWidget *create_top_window(void)
{
	char window_title[LINEWIDTH];
	GtkWidget *window;

	/* create window */
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	/* set attributes */
	snprintf(window_title, sizeof(window_title), "%s", 
		get_version_str()); 
	gtk_window_set_title(GTK_WINDOW(window), window_title);
	gtk_container_set_border_width(GTK_CONTAINER(window), NOSPACING);
	gtk_window_set_policy(GTK_WINDOW(window), NOSHRINK, NOGROW, NOAUTOGROW);
	gtk_window_set_wmclass(GTK_WINDOW(window), "umix", "Umix");
	
	/* connect signals */
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
		GTK_SIGNAL_FUNC(cb_quit_func), NULL);

	return window;
}

static void cb_quit_func(GtkButton *button, gpointer *data)
{
	gtk_main_quit();
}

int ui_gtk_check(void)
{
	if( gtk_init_check(NULL, NULL) == FALSE )
		return -1;
	else
		return 0;
}

void ui_gtk_exit(void)
{
#ifdef UMIX_DEBUG
	errormsg("ui_gtk_exit: GTK UI deinitialized");
#endif
}
